;
; Collater v2 beta. Requires AutoIt Beta
; Pete Witt.
;
; v2.0		01.03.06 Changed INI format: each computer has it's own section.
;					 Can supply Acct/Pwd for non-authenticated machines.
;					Added ability to view Windows' Events with 'File1=Events'
;					The ListView only shows the SysEvent.evt file details (though all evt files are accessed via EventVwr).
;
; v1.1		23.02.06 Added FileOpenDialog(): select INI, if none was supplied (via cmd line, or dropped on .exe)
;
;
; Purpose:
;					Allows multiple files from multiple locations to be displayed in one window.
;
; Simple listview display of: full file path, date, size.
;  The latter 2 are often useful quick-check of log/status files: is it up-to-date; is the file size correct (0 or non-0)?
;
; Uses an INI file to specify a list of files to display - wildcard indicates 'get the newest file matching this pattern'
;
; Single-click the file to open it; an alternative app can be specified for non-associated files.
;
#Region Compiler directives section
#Compiler_Res_Comment=Allows multiple files from multiple locations to be displayed in one window.
#Compiler_Res_Description=Remote Updater
; *UPDATE here* Full file version
#Compiler_Res_Fileversion=2.0.0.0
;#Compiler_Icon=au3_icon_1_red.ico                 ;filename of the Ico file to use
#Compiler_Res_Field1Name= Created By  ;Free format fieldname 1 (could be used for Copyright/Company/Programmer etc
#Compiler_Res_Field1Value= Pete Witt     ;Free format fieldvalue 1
#EndRegion
; >----------------------------------------
#include <GuiConstants.au3>
#include <GuiListView.au3>
; 2 vars reqd from Constants.au3
Global Const $STDOUT_CHILD = 2
Global Const $STDERR_CHILD = 4

Opt("RunErrorsFatal", 0) ;1=fatal, 0=silent set @error=1
Opt('MustDeclareVars', 1)

Global $AltApp, $RunAltApp = 0

Global $Result, $LastErr, $Debug
Global $MyTitle = "", $INIFile = "", $LogFile = ""
Global $Computer[1], $Acct[1], $Pwd[1];ReDimmed in ParseINI, once nComputers is known.

; GUI stuff
Global Const $WindowWidth = @DesktopWidth / 2
Global Const $WindowHeight = @DesktopHeight / 2
Global Const $WindowX = @DesktopWidth / 4
Global Const $WindowY = @DesktopHeight / 4
Global Const $Margin = 10
Global $FileEntry[1]
Global $EventMode = 0

_Main()
Exit; Not reqd

Func _Main()
	Local $listview, $Btn_Exit, $msg, $Status, $i, $Col2, $Col3, $FileTime
	Local $Tip1, $Tip2, $SrvrVol
	;Local $MyTitle = "Collater"
	
	$Debug = 0; Change to 1 for Debug messages
	$INIFile = ""
	_CmdLine()
	
	If $INIFile = "" Then
		; $INIFile = @ScriptDir & "\" & StringTrimRight(@ScriptName, 3) & "Ini"
		$INIFile = FileOpenDialog("Open file Dialog Box", @ScriptDir & "\", "INI Files (*.ini)", 3) ;,@ScriptFullPath)
		If @error Then Exit; User cancelled
	Else
		; An INI name was supplied; maybe missing the Script's dir prefix?
		If FileExists($INIFile) = 0 Then $INIFile = @ScriptDir & "\" & $INIFile
	EndIf
	
	If FileExists($INIFile) = 0 Then
		MsgBox(16, "Collater", $INIFile & " not found - aborting...", 10)
		Exit
	Else
		; Initially set MyTitle to INI name, to allow multiple Collater .exes (using different INIs)
		$MyTitle = StringTrimRight(StringTrimLeft($INIFile, StringInStr($INIFile, "\", 0, -1)), 4); 0 case insensitive, -1 find LAST occurrence of \
		; Prevent Duplicate Script From Running
		If WinExists("AutoIt " & $MyTitle) Then Exit ; It's already running
		AutoItWinSetTitle("AutoIt " & $MyTitle)
		_ParseINI()
	EndIf
	
	AdlibEnable("_MyAdLib") ; See _MyAdLib at end of script - deal with non-associated files.
	
	Local $LVItem[UBound($FileEntry) ]; Need to store each file's line in Listview
	; GUI stuff
	
	GUICreate($MyTitle, $WindowWidth, $WindowHeight + $Margin, $WindowX, $WindowY, $WS_SIZEBOX + $WS_SYSMENU)
	$listview = GUICtrlCreateListView("File|Date (YMD) Time|Size (KB)", $Margin, $Margin, $WindowWidth- (2 * $Margin), $WindowHeight- (6 * $Margin), BitOR($LVS_REPORT, $LVS_SINGLESEL), BitOR($LVS_EX_GRIDLINES, $LVS_EX_FULLROWSELECT))
	GUICtrlSetResizing($listview, $GUI_DOCKAUTO) ; Handle listview area automatically
	
	For $i = 0 To UBound($FileEntry) - 1
		If $FileEntry[$i] <> "" Then
			$LVItem[$i] = GUICtrlCreateListViewItem($FileEntry[$i] & "|...|...", $listview) ; Write the 1st column
		EndIf
	Next
	
	; Column widths
	_GUICtrlListViewSetColumnWidth ($listview, 0, $LVSCW_AUTOSIZE) ; Give UNC path the lion's share
	_GUICtrlListViewSetColumnWidth ($listview, 0, _GUICtrlListViewGetColumnWidth ($listview, 0) + $Margin) ; Make it a little bigger (Autowidth+Margin)
	
	Local $WidthRemaining = $WindowWidth- (2 * $Margin) - 4- (_GUICtrlListViewGetColumnWidth ($listview, 0))
	; Give the date/time & size colums 1/2 of what's left
	_GUICtrlListViewSetColumnWidth ($listview, 1, $WidthRemaining / 2)
	_GUICtrlListViewSetColumnWidth ($listview, 2, $WidthRemaining / 2)
	
	; Labels & a button
	Local $StatusInfo = "Retrieving file information..."
	$Status = GUICtrlCreateLabel($StatusInfo, $Margin, $WindowHeight- (4 * $Margin), 20 * $Margin, 2 * $Margin);, $SS_SUNKEN)
	GUICtrlSetResizing($Status, $GUI_DOCKBOTTOM + $GUI_DOCKLEFT + $GUI_DOCKHEIGHT + $GUI_DOCKWIDTH); Keep label on left at bottom, don't resize it
	
	; On-screen tips - create 'em hidden, show em later
	$Tip1 = GUICtrlCreateLabel("Click column heading to sort", $WindowWidth- (20 * $Margin) - $Margin, $WindowHeight- (4 * $Margin) - $Margin, 20 * $Margin, 2 * $Margin, $SS_RIGHT);, $SS_SUNKEN)
	GUICtrlSetResizing($Tip1, $GUI_DOCKBOTTOM + $GUI_DOCKLEFT + $GUI_DOCKHEIGHT + $GUI_DOCKWIDTH); Keep label on left at bottom, don't resize it
	GUICtrlSetState($Tip1, $GUI_HIDE)
	
	$Tip2 = GUICtrlCreateLabel("Click row to open file", $WindowWidth- (20 * $Margin) - $Margin, $WindowHeight- (3 * $Margin), 20 * $Margin, 2 * $Margin, $SS_RIGHT);, $SS_SUNKEN)
	GUICtrlSetResizing($Tip2, $GUI_DOCKBOTTOM + $GUI_DOCKLEFT + $GUI_DOCKHEIGHT + $GUI_DOCKWIDTH); Keep label on left at bottom, don't resize it
	GUICtrlSetState($Tip2, $GUI_HIDE)
	
	$Btn_Exit = GUICtrlCreateButton("Exit", ($WindowWidth / 2) - (2 * $Margin), $WindowHeight- (4 * $Margin), 4 * $Margin, 2 * $Margin)
	;GUICtrlSetResizing ( $Btn_Exit,$GUI_DOCKAUTO); $GUI_DOCKBOTTOM+$GUI_DOCKSIZE+$GUI_DOCKVCENTER)
	GUICtrlSetResizing($Btn_Exit, $GUI_DOCKBOTTOM + $GUI_DOCKVCENTER + $GUI_DOCKHEIGHT + $GUI_DOCKWIDTH); Keep btn centred at bottom, don't resize it
	
	GUISetState(@SW_SHOW)
	
	; Grab the info - populate the dialogue...
	$StatusInfo = "Done"
	Local $j, $FileType, $ListEntry = 0
	Local $Authenticate[UBound($Computer) ]
	
	For $i = 0 To UBound($Computer) - 1
		$msg = GUIGetMsg()
		If ($msg = $GUI_EVENT_CLOSE Or $msg = $Btn_Exit) Then
			$StatusInfo = "Cancelled"
			ExitLoop
		EndIf
		; Only connect & disconnect (via net use) if we have Acct/Pwd info & not already authenticated.
		$Authenticate[$i] = 1 - FileExists($FileEntry[$ListEntry]) ; Does the 1st file listed for each computer exist? If not, assume we need to auth.
		If $Authenticate[$i] Then
			If $Acct[$i] = "" Or $Pwd[$i] = "" Then; Can't authenticate if no Acct/Pwd
				$Authenticate[$i] = 0
			Else
				$msg = "Connecting to " & $Computer[$i]
				If $Debug Then _DebugMsg("", $msg)
				$Col2 = "Connecting..."
				GUICtrlSetData($LVItem[$ListEntry], "|" & $Col2)
				$Result = _MyNetUse("connect", $Computer[$i], $Acct[$i], $Pwd[$i])
				If $Result = 0 Then
					If $Debug Then _DebugMsg("", "Failed to connect to:" & $Computer[$i], 2000)
					ContinueLoop; Connect failed, can't do owt with this computer - try the next
				EndIf
			EndIf
		EndIf
		Do
			$SrvrVol = StringLeft($FileEntry[$ListEntry], StringInStr($FileEntry[$ListEntry], "\", 0, 4)) ; Extract \\Machine\'Volume' or 'Share' from full UNC
			If DriveStatus($SrvrVol) = "READY" Then
				;------------------------------------------------------------------------------
				$FileType = _FileType($FileEntry[$ListEntry])
				If $FileType = 2 Then; Wildcard was supplied
					$FileEntry[$ListEntry] = _GetNewestFile($FileEntry[$ListEntry]); Get newest filename from supplied wildcard
					$FileType = _FileType($FileEntry[$ListEntry]) ; Confirm this (single) file exists...
					If $FileType = 1 Then GUICtrlSetData($LVItem[$ListEntry], $FileEntry[$ListEntry]) ; Update the 1st column, if file was found
				EndIf
				If $FileType = 1 Then;	Single files exists
					$FileTime = FileGetTime($FileEntry[$ListEntry], 0, 0) ; Return modified date as an array
					If Not @error Then
						$Col2 = $FileTime[0] & "." & $FileTime[1] & "." & $FileTime[2] & "  " & $FileTime[3] & ":" & $FileTime[4]
						$Col3 = Round(FileGetSize($FileEntry[$ListEntry]) / 1024, 1) ; Bytes->KB
					Else
						$Col2 = "Date error"
						$Col3 = "N/A"
					EndIf
				Else
					$Col2 = "File not found."
					$Col3 = "N/A"
				EndIf
			Else
				$Col2 = "UNC path '" & $SrvrVol & "' not found."
				$Col3 = "N/A"
			EndIf
			GUICtrlSetData($LVItem[$ListEntry], "|" & $Col2 & "|" & $Col3)
			$ListEntry = $ListEntry + 1
			If _GetNameFromUNC($FileEntry[$ListEntry]) <> $Computer[$i] Then ExitLoop; Move onto next PC
		Until $ListEntry > UBound($FileEntry)
		
		$msg = GUIGetMsg()
		If ($msg = $GUI_EVENT_CLOSE Or $msg = $Btn_Exit) Then
			$StatusInfo = "Cancelled"
			ExitLoop
		EndIf
		
	Next
	
	If $StatusInfo = "Cancelled" Then
		For $j = $i + 1 To UBound($FileEntry) - 1
			GUICtrlSetData($LVItem[$j], "|Cancelled|N/A") ; Set Col3 (Size) to N/A - used to prevent attempt to open file
		Next
	EndIf
	
	GUICtrlSetData($Status, $StatusInfo)
	Sleep(2000) ; Display status for a couple of secs
	GUICtrlSetState($Status, $GUI_HIDE)
	
	GUICtrlSetState($Tip1, $GUI_SHOW)
	GUICtrlSetState($Tip2, $GUI_SHOW)
	
	; If using a variable no. of columns...
	; Local $nCols=_GUICtrlListViewGetSubItemsCount ($listview,$MyTitle)
	
	GUISetState()
	Local $B_DESCENDING[3] ; $ncols, No. of columns
	; Note re: _GUICtrlListViewSort - once you've re-ordered a list, 1st item has a new ID! e.g. here, it's (old) LastItem+2 (label and exit button)
	; Re-wrote section, base on _GUICtrlListViewGetItemText help info.
	Local $ListRow, $s_StartPath, $pos
	
	; Dialog event loop
	While 1
		$msg = GUIGetMsg()
		Select
			Case $msg = $GUI_EVENT_CLOSE Or $msg = $Btn_Exit
				ExitLoop
			Case $msg = $listview
				_GUICtrlListViewSort ($listview, $B_DESCENDING, GUICtrlGetState($listview)); sort by the column header clicked on
			Case $msg = $GUI_EVENT_PRIMARYDOWN
				$pos = GUIGetCursorInfo()
				If ($pos[4] == $listview) Then; Cursor is hovering over a $listview control...
					$ListRow = _GUICtrlListViewGetCurSel ($listview) ; Index of current selectiob (-ve if non-listview item selected)
					If $Debug Then _DebugMsg("tip", "Current Index Selected: " & _GUICtrlListViewGetCurSel ($listview) & "," & _GUICtrlListViewGetItemText ($listview, $ListRow, 0), 3000)
					If $ListRow >= 0 And _GUICtrlListViewGetItemText ($listview, $ListRow, 2) <> "N/A" Then; A list entry was selected, but last column mustn't be 'N/A' (file hasn't been accessed)
						$s_StartPath = _GUICtrlListViewGetItemText ($listview, $ListRow, 0) ;
						$RunAltApp = 0
						If $EventMode = 0 Then
							_StartFile($s_StartPath) ; Use Windows' Start command to open file by association
						Else
							; Run Event Viewer mmc [AltApp was set appropriately in _ParseINI()].
							If $Debug Then _DebugMsg("", $AltApp & " /computer=" & _GetNameFromUNC($s_StartPath), 1000)
							Run($AltApp & " /computer=" & _GetNameFromUNC($s_StartPath));,@SW_HIDE) ; StringMid($s_StartPath, 3, StringInStr($s_StartPath, "\", 0, 3) - 3)
						EndIf
					EndIf
				EndIf
		EndSelect
		; $RunAltApp is set=1 in _AdLib - only if _StartFile() tried to open a file that has no associated app.
		If $RunAltApp = 1 Then
			If $Debug Then _DebugMsg("", "$RunAltApp:" & $RunAltApp & ",EventMode:" & $EventMode & $AltApp & ' "' & $s_StartPath & '"')
			$RunAltApp = 0
			If $s_StartPath <> "" And $EventMode <> 1 Then Run($AltApp & ' "' & $s_StartPath & '"', '')
		EndIf
	WEnd
	
	; Before quitting, disconnect from those machines we previosuly connected to...
	For $i = 0 To UBound($Computer) - 1
		If $Authenticate[$i] Then
			$msg = "Disconnecting from " & $Computer[$i]
			If $Debug Then _DebugMsg("", $msg)
			;If $Debug Then _DebugMsg("",$Msg)
			$Result = _MyNetUse("disconnect", $Computer[$i], $Acct[$i], $Pwd[$i])
			If $Result = 0 Then
				$msg = "ERROR: Failed to disconnect."
				Sleep(500)
			EndIf
		EndIf
	Next
	
EndFunc   ;==>_Main

Func _ParseINI()
	Local $iData, $iSection
	Local $SectionName, $SectionData, $tmpKey, $nComputers = 0, $Current
	Local $UpdateINI, $ListEntry = 0
	
	$SectionName = IniReadSectionNames($INIFile) ; Store Sections in $SectionName[]
	If @error Or ($SectionName[0] < 2 Or StringLower($SectionName[1])<>"common") Then
		_DebugMsg("", " ERROR: Can't find correct Sections - Must have [Common] and at least 1 [Computer'N'] - aborting.")
		Exit
	Else
		$MyTitle = IniRead($INIFile, "Common", "About", $MyTitle) ; MyTitle is initially the .INI filename; update to About data (if available)
		$AltApp = IniRead($INIFile, "Common", "AltApp", "")
		If StringLen($AltApp)>0 And StringLeft($AltApp, 1) <> '"' Then $AltApp = '"' & $AltApp & '"'	; Wrap in quotes
		For $iSection = 2 To $SectionName[0]
			If StringLower(StringLeft($SectionName[$iSection], 8)) = "computer" And StringIsDigit(StringTrimLeft($SectionName[$iSection], 8)) Then
				$nComputers = $nComputers + 1; Count number of [ComputerN] sections
			EndIf
		Next
		
		ReDim $Computer[$nComputers], $Acct[$nComputers], $Pwd[$nComputers]
		; Parse each [ComputerN] section...
		$Current = 0 ;
		For $iSection = 2 To $SectionName[0]
			; Do 1 or more [ComputerN] sections
			If StringLower(StringLeft($SectionName[$iSection], 8)) = "computer" And StringIsDigit(StringTrimLeft($SectionName[$iSection], 8)) Then
				$UpdateINI = 0
				$SectionData = IniReadSection($INIFile, $SectionName[$iSection]); Store Keys in $SectionData[][0] and their values in $SectionData[][1]
				If IsArray($SectionData) = 0 Then
					If $Debug Then _DebugMsg("", "[" & $SectionName[$iSection] & "] Is empty", 1000)
					ContinueLoop ; Skip empty [ComputerN] section (& don't inc $Current)
				EndIf
				$Computer[$Current] = ""
				For $iData = 1 To $SectionData[0][0]; Read thru keys
					$tmpKey = StringLower($SectionData[$iData][0])
					Select
						Case $tmpKey = "name"
							If $Computer[$Current] <> "" Then; User supplied>1 Name
								_DebugMsg("", "ERROR: Multiple computer names in [" & $SectionName[$iSection] & "]", 1000)
								Exit
							Else
								$Computer[$Current] = StringLower($SectionData[$iData][1])
								If StringLeft($Computer[$Current], 2) = "\\" Then $Computer[$Current] = StringTrimLeft($Computer[$Current], 2)
							EndIf
						Case StringLeft($tmpKey, 4) = "file"
							; Amend FileN entries, as they appear in FileListView dialog
							If StringLower($SectionData[$iData][1]) <> "events" Then
								If StringLeft($SectionData[$iData][1], 1) = "\" Then $SectionData[$iData][1] = StringTrimLeft($SectionData[$iData][1], 1)
								$FileEntry[$ListEntry] = "\\" & $Computer[$Current] & "\" & $SectionData[$iData][1]
							Else
								$FileEntry[$ListEntry] = "\\" & $Computer[$Current] & "\Admin$\System32\Config\SysEvent.evt"
								;Got to pick one event file - SysEvent is generally most frequently written-to.
								$AltApp = @SystemDir & "\mmc " & @SystemDir & "\EventVwr.msc"
								$EventMode = 1
							EndIf
							; _DebugMsg("",$Computer[$Current] & ",File:" & $FileEntry[$ListEntry])
							$ListEntry = $ListEntry + 1
							ReDim $FileEntry[$ListEntry + 1]
						Case $tmpKey = "acct"
							; The Acct key exists: either an acct is specified (encrypted) or is reqd.
							; If the key doesn't exist, assume we're using pass-thru credentials (ie the current user)
							; To distinguish between no Acct key ['use my credentials'] & empty value [input data to be encrypted]- special case Input
							$Acct[$Current] = $SectionData[$iData][1]
							If $Acct[$Current] = "" Then
								$Acct[$Current] = "I"
								$Pwd[$Current] = "I"; If setting Acct, *must* also set it's pwd
								$UpdateINI = 1; Need to update INI file
							Else
								$Acct[$Current] = _MyDecrypt($Acct[$Current])
							EndIf
						Case $tmpKey = "pwd"
							$Pwd[$Current] = $SectionData[$iData][1]
							; Need to distinguish between no Acct key [use my credentials] & empty value [input data to be encrypted]- special case Input
							If $Pwd[$Current] = "" Then
								$Pwd[$Current] = "I"; If acct already set, may only need to update it's pwd
								$UpdateINI = 1; Need to update INI file
							Else
								$Pwd[$Current] = _MyDecrypt($Pwd[$Current])
							EndIf
					EndSelect
				Next
				;_DebugMsg("",$Computer[$Current] & ", Acct,Pwd:" & $Acct[$Current] & "," & $Pwd[$Current])
				If $Computer[$Current] = "" Then
					_DebugMsg("", "[" & $SectionName[$iSection] & "] Name key missing")
				Else
					; >----- Acct/Pwd
					If $UpdateINI = 1 Then; We need to enter acct/pwd
						_UpdateCredentials($INIFile, $SectionName[$iSection], $Computer[$Current], $Acct[$Current], $Pwd[$Current])
					EndIf
				EndIf;	Not a blank computer name
				$Current = $Current + 1; In case there's some commented-out/badly-named sections - keep an independent track of Current proper section
			EndIf
		Next;$iSection [ComputerN]
	EndIf
EndFunc   ;==>_ParseINI

Func _DebugMsg($strMsgType, $strMsg, $nDelay = 0)
	; Help with debugging
	; MsgBox flags: 16 Stop-sign icon 0x10; 32 Question-mark icon 0x20; 48 Exclamation-point icon 0x30; 64 Icon consisting of an 'i' in a circle
	If StringLower($strMsgType) = "tip" Then
		ToolTip($strMsg, 0, 0)
	Else
		MsgBox(4096 + 48, $MyTitle & " - Debug", $strMsg)
	EndIf
	Sleep($nDelay)
EndFunc   ;==>_DebugMsg

Func _UpdateCredentials($INIFile, $Section, $ThisComputer, ByRef $ThisAcct, ByRef $ThisPwd)
	; Called from ParseINI
	; Uses Globals: $Acct[], $Pwd[], $MyTitle, $WindowWidth, $WindowHeight, $WindowX, $WindowY, $INIFile, $Debug, _DebugMsg()
	; Uses Functions: _MyEncrypt(), _MyDecrypt(), _DebugMsg()
	Local $UpdateINI; May need to update the INI file - when setting (encrypted) Acct/Pwd fields
	Local $tmpAcct, $tmpPwd, $msg
	
	;_DebugMsg("Collater Warning"," Section [" & $Section & "] needs to be updated.")
	If $ThisAcct = "I" Then; present type acct dialog & save info to be written at end.
		$msg = " Authentication account required for computer: " & StringUpper($ThisComputer)
		_DebugMsg("", $msg)
		$tmpAcct = ""
		Do
			$tmpAcct = InputBox($MyTitle, $msg & @CRLF, "", " M", $WindowWidth, $WindowHeight / 4, $WindowX, $WindowY+ ($WindowHeight / 2)) ; Visible, Mandatory
			$LastErr = @error
		Until $tmpAcct <> "" Or $LastErr <> 0
		If $tmpAcct = "" Then
			; log that you quit, save log file & exit
			_DebugMsg("", " User quit account entry dialog - aborting.", 2000)
			Exit
		EndIf
		$ThisAcct = $tmpAcct
		$tmpAcct = _MyEncrypt($tmpAcct)
		;msgbox(0,"",$INIFile , $SectionName[$iSection], $Acct[$Current], $tmpAcct
		$UpdateINI = IniWrite($INIFile, $Section, "Acct", $tmpAcct) ; Sets UpdateINI to 1 (OK) or 0 (bad)
		If $UpdateINI <> 1 Then
			If $Debug Then _DebugMsg("tip", "Failed updating INI", 1000)
			$msg = " ERROR: Failed to update INI file with new authentication account details - aborting."
			_DebugMsg("", $msg, 2)
			Exit
		EndIf
	EndIf
	
	If $ThisPwd = "I" Then; present type acct dialog & save info to be written at end.
		$msg = " Password required for '" & $ThisAcct & "' account on computer: " & StringUpper($ThisComputer)
		_DebugMsg("", $msg)
		$tmpPwd = ""
		Local $tmpPwd1 = ""
		Do
			$tmpPwd = InputBox($MyTitle, $msg, "", "M", $WindowWidth, $WindowHeight / 4, $WindowX, $WindowY+ ($WindowHeight / 2)) ; Hidden, Mandatory. Alt-7 disguises text
			$LastErr = @error
			If $tmpPwd <> "" Then
				; We entered something -
				If $tmpPwd1 = "" Then;Re-enter to confirm
					$tmpPwd1 = $tmpPwd
					$tmpPwd = ""
					$msg = " Re-type to confirm..."
				Else
					If $tmpPwd == $tmpPwd1 Then; Need to test 'pwds don't match', but with string compares <> and = are case-insensitive! However, == isn't.
						$ThisPwd = $tmpPwd
						$tmpPwd = _MyEncrypt($tmpPwd)
						; DON'T do this under std Debug conditions - it'll display plain-text acct & pwd
						;_DebugMsg("tip", "Pwds match " & $tmpPwd & " " & $tmpPwd1 & " " & $LastErr, 500)
						$UpdateINI = IniWrite($INIFile, $Section, "Pwd", $tmpPwd) ; Sets UpdateINI to 1 (OK) or 0 (bad)
						If $UpdateINI <> 1 Then
							If $Debug Then _DebugMsg("tip", "Failed updating INI", 1000)
							$msg = " ERROR: Failed to update INI file with new authentication account details - aborting."
							_DebugMsg("", $msg, 2)
							Exit
						EndIf
					Else
						$tmpPwd = ""
						$tmpPwd1 = ""
						$msg = "The passwords don't match - please start again."
					EndIf
				EndIf
			EndIf
		Until $tmpPwd <> "" Or $LastErr <> 0
		
		If $tmpPwd = "" Or $tmpPwd1 = "" Then
			; If we've just written a new Acct, but then failed to set its pwd, must blank pwd.
			If $UpdateINI = 1 Then $UpdateINI = IniWrite($INIFile, $Section, "Pwd", "")
			If $Debug Then _DebugMsg("tip", "Pwd problem", 500)
			$msg = " User quit password entry dialog - aborting."
			_DebugMsg("", $msg, 2)
			Exit
		EndIf
	Else
		;msgbox(0,"Debug","Pwd decrypted pwd:" & $Pwd[$Current])
	EndIf
	
EndFunc   ;==>_UpdateCredentials

Func _StartFile($s_StartPath)
	; To do: determine if there's no associated app & try opening it in NotePad (or default reader e.g. UltraEdit)
	Local $s_StartStr
	If @OSTYPE = 'WIN32_NT' Then
		$s_StartStr = @ComSpec & ' /c start /B "" '; XP /B - start app without creating a new window.
	Else
		$s_StartStr = @ComSpec & ' /c start '
	EndIf
	Run($s_StartStr & '"' & $s_StartPath & '"', '', @SW_HIDE)
EndFunc   ;==>_StartFile

Func _CmdLine() ; Read command line options
	For $i = 1 To UBound($CmdLine) - 1
		Select
			Case $CmdLine[$i] = "?" Or $CmdLine[$i] = "/?" Or $CmdLine[$i] = "help" Or $CmdLine[$i] = "/help"
				MsgBox(0, "Help", "/? - Displays this message" & @LF & _
						"filename        - Specify file to read instructions from (Silent mode)" & @LF);& _ 
				Exit
			Case StringRight($CmdLine[$i], 4) = ".ini"
				$INIFile = $CmdLine[$i]
		EndSelect
	Next
EndFunc   ;==>_CmdLine

Func _FileType($strFile)
	;	Returns 0 not found; 1 found single file; 2 wildcard; 3 folder
	
	Local $FileType = 0
	If StringInStr($strFile, "*") Or StringInStr($strFile, "?") Then
		$FileType = 2 ; File maybe OK, uses wildcards
		$strFile = StringLeft($strFile, StringInStr($strFile, "\", 0, -1)) ; Upto last \
		
		If _IsFolder($strFile) = 0 Then
			$FileType = 0
		EndIf
	Else
		If _IsFolder($strFile) Then
			$FileType = 3; Will need to recurse this $File[]
		Else
			; Not a folder. Are wildcards used, or is it a single file
			If @error <> 1 Then
				$FileType = 1
				;Single File was found
			Else
				;If $Debug Then _DebugMsg("tip", "File not found " & $strFile & ".", 1000)
			EndIf
			; Don't do a recursive list
		EndIf
	EndIf
	
	Return $FileType
EndFunc   ;==>_FileType

Func _IsFolder($sPath)
	; erifash's 'Is it a folder?...', http://www.autoitscript.com/forum/index.php?showtopic=17662&hl=
	; Return Path is folder: 1, Path is file: 0
	; If File/Folder doesn't exist returns 0 and sets error=1
	If Not FileExists($sPath) Then
		SetError(1)
		Return 0
	EndIf
	Return Not Not StringInStr(FileGetAttrib($sPath), "D")
EndFunc   ;==>_IsFolder

Func	_GetNewestFile ($strPath)
	; Derived from Larry's 'New File Search function'; http://www.autoitscript.com/forum/index.php?showtopic=2753&hl=
	; Gets newest file (by modified date) in a dir, returns full file path
	Local $hFile, $szBuffer, $szInfo
	Local $szInfoN, $strNewest, $strRootPath, $strFName
	
	; Split path\wildcard into Path and Filename/wildcard
	;Returns just the text after the final \ i.e. normally the filename/mask. If no \, returns string unaltered.
	$strFName = StringTrimLeft($strPath, StringInStr($strPath, "\", 0, -1)); 0 case insensitive, -1 find LAST occurrence of \
	$strRootPath = StringTrimRight($strPath, StringLen($strFName))
	
	$hFile = FileFindFirstFile($strRootPath & $strFName)
	
	If $hFile >= 0 Then
		$szBuffer = FileFindNextFile($hFile)
		$szInfo = FileGetTime($strRootPath & $szBuffer, 0, 1); Get 1st file's 'Modified' date as a string
		While Not @error
			If $szBuffer <> "." And $szBuffer <> ".." And StringInStr(FileGetAttrib($strRootPath & $szBuffer), "D") = 0 Then; Only check files
				$szInfoN = FileGetTime($strRootPath & $szBuffer, 0, 1); Return 'Modified' date as a string
				If $szInfoN > $szInfo Then
					$szInfo = $szInfoN
					$strNewest = $strRootPath & $szBuffer
				EndIf
			EndIf
			$szBuffer = FileFindNextFile($hFile)
		WEnd
		FileClose($hFile)
	EndIf
	Return $strNewest
EndFunc   ;==>_GetNewestFile 

Func _MyEncrypt($Text)
	; Simple encryption method - convert plain text to encrypted hex string, with a scramble
	; Derived from datkewlguy's 'ABC Encryption' http://www.autoitscript.com/forum/index.php?showtopic=11973&hl=
	Local $tEncrypt = "", $LenText, $i, $Scrambled = "", $HexSecret
	$LenText = StringLen($Text)
	
	If $LenText > 0 Then
		For $i = 1 To 2; Scramble it twice
			$Scrambled = ""
			For $i1 = 1 To Int($LenText / 2)
				$Scrambled = $Scrambled & StringMid($Text, $LenText - $i1 + 1, 1) & StringMid($Text, $i1, 1)
			Next
			If Mod($LenText, 2) Then; Pick up the odd character.
				$Scrambled = $Scrambled & StringMid($Text, $i1, 1)
			EndIf
			$Text = $Scrambled
		Next
		For $i = 1 To $LenText
			$HexSecret = Hex( Mod( Asc( StringMid($Text, $i, 1)) + 128, 256), 2) ; Shift chr 1/2 way round alphabet. Get 2chr Hex.
			If Mod(TimerInit(), 2) Then $HexSecret = StringLower($HexSecret) ; Pseudo-random upper/lower case just to make appearance more unusual!
			$tEncrypt = $tEncrypt & $HexSecret
		Next
	EndIf
	Return $tEncrypt
	
EndFunc   ;==>_MyEncrypt

Func _MyDecrypt($Text)
	; Simple decryption method - convert encrypted hex string to plain text, with a scramble
	Local $tDecrypt = "", $LenText, $DecText, $i, $Unscrambled1 = "", $Unscrambled2 = ""
	$LenText = StringLen($Text)
	If ($LenText > 0) And StringIsXDigit($Text) And (Mod($LenText, 2) = 0) Then
		; Text length *must* be an even no. i.e. Integer div, & must be decrypting a Hex string [0-9,A-F]
		For $i = 1 To Int($LenText / 2) ; Do 2 Hex chrs at a time
			$DecText = Dec(StringMid($Text, ($i * 2) - 1, 2)) ; Convert Hex to Dec
			$tDecrypt = $tDecrypt & Chr(Mod($DecText + 128, 256))
		Next
		$LenText = StringLen($tDecrypt)
		; UnScramble it twice
		For $i = 1 To 2
			$Unscrambled1 = ""
			$Unscrambled2 = ""
			For $i1 = 1 To $LenText Step 2
				$Unscrambled1 = StringMid($tDecrypt, $i1, 1) & $Unscrambled1
				$Unscrambled2 = $Unscrambled2 & StringMid($tDecrypt, $i1 + 1, 1)
			Next; $i1
			If Mod($LenText, 2) Then; Pick up the odd character.
				$Unscrambled1 = StringMid($tDecrypt, $i1, 1) & $Unscrambled1
			EndIf
			$tDecrypt = $Unscrambled2 & $Unscrambled1
		Next
	EndIf
	
	Return $tDecrypt
EndFunc   ;==>_MyDecrypt

Func _GetNameFromUNC($strUNC)
	If StringLeft($strUNC, 2) = "\\" And StringLen($strUNC > 3) Then
		Return StringMid($strUNC, 3, StringInStr($strUNC, "\", 0, 3) - 3) ; Determine ComputerName from UNC Path
	Else
		SetError(1)
		Return ""
	EndIf
EndFunc   ;==>_GetNameFromUNC

Func _MyNetUse($Cmd, $Computer, $Acct = "", $Pwd = "")
	; Cmd is either connect or disconnect. Returns 1 OK, 0 on error
	
	; Authenticate as reqd: 'net use' will use pass-thru authentication where possible (for both Windows & Netware [with client s/ware installed])
	; Netware: though authenticated to a Tree, need to ensure we're autheticated to (remote) server. Do this by a 'net use' to access its resources
	; Windows: no real need to do this, just simpler. Creates an IPC$ connection
	; Where servers are in a different tree, domain/workgroup, need to use acct/pwd.
	; Delete these connections when done.
	
	If $Computer = "" Then Return 0
	If StringLeft($Computer, 2) <> "\\" Then $Computer = "\\" & $Computer
	Local $NetUseCmd, $CmdLine, $AllStdOut, $StdOutText
	
	$NetUseCmd = "net use " & $Computer
	$Cmd = StringLower($Cmd)
	$CmdLine = ""
	
	If $Cmd = "disconnect" Then
		$CmdLine = " /delete"; Don't set Credentials if disconnecting
	Else
		If $Acct <> "" Then; Only supply acct/pwd if reqd. (see above)
			$CmdLine = " /user:" & $Acct & " " & $Pwd
		EndIf
	EndIf
	; DON'T do this under std Debug conditions - it'll display plain-text acct & pwd
	;_MyMsgBox("Run()",$NetUseCmd & $CmdLine)
	
	;$AllStdOut = Run($NetUseCmd & $CmdLine, "", @SW_HIDE, $STDERR_CHILD + $STDOUT_CHILD)
	$AllStdOut = Run(@ComSpec & " /c " & $NetUseCmd & $CmdLine, "", @SW_HIDE, $STDERR_CHILD + $STDOUT_CHILD)
	$StdOutText = _GetAllStdOut($AllStdOut)
	
	; error egs: 67 network name not found [takes ages!]
	; System error 1219 has occurred. (Already connected).
	;	System error 1326 has occurred. Logon failure: unknown user name or bad password.
	; connects OK: The command completed successfully. /delete gives: \\symposium was deleted successfully.
	If StringInStr($StdOutText, "system error") Then
		If $Debug Then _DebugMsg("tip", "ERROR: " & $NetUseCmd & " failed to " & $Cmd & @CR & $StdOutText, 3000)
		Return 0; Can't do owt with this computer
	Else
		If $Debug Then _DebugMsg("tip", $NetUseCmd & " succeeded on " & $Cmd & @CR & $StdOutText, 3000)
		Return 1
	EndIf
EndFunc   ;==>_MyNetUse

Func _GetAllStdOut($strStdOut)
	; Returns all StdOut & Err info as a single string
	Local $StdOutText = ""
	Local $StdErrText = ""
	While 1
		$StdOutText = $StdOutText & StdoutRead($strStdOut)
		If @error = -1 Then ExitLoop
	WEnd
	While 1
		$StdErrText = $StdErrText & StderrRead($strStdOut)
		If @error = -1 Then ExitLoop
	WEnd
	Return ($StdOutText & $StdErrText)
EndFunc   ;==>_GetAllStdOut

Func _MyAdLib()
	If WinExists("Windows", "Windows cannot open this file") Then
		WinActivate("Windows", "Windows cannot open this file")
		Sleep(50)
		Send("{DOWN 4}") ; Cursor down Cancel btn
		Send("{ENTER}") ; Enter
		If $AltApp <> "" Then $RunAltApp = 1; AdLib determined that the file was not associated, open with AltApp (if specified)
	EndIf
EndFunc   ;==>_MyAdLib
;EOF

